import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import requests
from PIL import Image
from wordcloud import WordCloud, STOPWORDS
from twitterpersona.sentiment_analysis import count_tweets
import urllib.request

def transform_zeros(val):
    if val == 0: 
       return 255
    else:
       return val

def create_wordcloud(df):
    """
    Create a wordcloud out of all the tweets of a user. 
    Wordcloud colour is green if the overal sentiment of the
    users tweets is positive, red if negative and blue if 
    neutral.

    Parameters
    ----------
    df : pd.DataFrame
        A dataframe with all the tweets of a user, and their
        sentiment analysis.
    
    Returns
    -------
    plot : plt.figure()
        A matplotlib plot of the wordcloud.
    
    Examples
    --------
    create_wordcloud(df)
    """

    # Import image from URL
    URL = 'https://www.edigitalagency.com.au/wp-content/uploads/twitter-logo-black-png.png'

    with urllib.request.urlopen(URL) as url:
        img = Image.open(url)
        mask = np.array(img)

    # Transform image to usable mask
    maskable_image = np.ndarray((mask.shape[0],mask.shape[1]), np.int32)
    for i in range(len(mask)):
        maskable_image[i] = list(map(transform_zeros, mask[i]))
    mask = maskable_image
    # Combine all tweets into single string
    text = " ".join(tweet for tweet in df["text_clean"])

    # Determine tweet sentiment
    sentiment = count_tweets(df)

    # Set wordcloud colour
    if max(sentiment, key=sentiment.get) == 'positive':
        colormap = 'summer'
    elif max(sentiment, key=sentiment.get) == 'negative':
        colormap = 'autumn'
    else:
        colormap = 'cool'

    # Create the wordcloud
    wordcloud = WordCloud(
        width = 800, 
        height = 800, 
        min_font_size = 10, 
        stopwords = STOPWORDS,
        mask = mask,
        background_color = 'white',
        contour_color = '#00aced',
        contour_width = 8,
        colormap = colormap
    ).generate(text)
    fig = plt.figure(figsize = (8, 8), facecolor = None)
    plt.imshow(wordcloud)
    plt.axis("off")
    plt.tight_layout(pad = 0)
    plt.show()
    
    return fig



import pandas as pd
import preprocessor as p
import nltk
from nltk.corpus import stopwords


def generalPreprocessing(df: pd.DataFrame) -> pd.DataFrame:
    '''
    Perform general preprocessing on df. Removes retweets/favourites and cleans URLs, Mentions, Numbers, and stop words.  
    
    Parameters
    ----------
    df : pd.DataFrame
        A dataframe storing all the raw data with text column.
    
    output_path : str
        the path that the newly generated csv should located at.
    
    Returns
    -------
    df : pd.DataFrame
        The processed tweet dataframe.
        
    Examples
    --------
    generalPreprocessing(df)
    '''

    # remove retweets/favourites
    rt_fav_pattern = r'\b(RT|FAV)\b'
    filter = df['text'].str.contains(rt_fav_pattern, regex=True)
    df = df[~filter]

    # remove URL, Mentions, and Numbers
    p.set_options(p.OPT.URL, p.OPT.MENTION, p.OPT.NUMBER)
    df['text_clean'] = df['text'].apply(lambda x: p.clean(x))

    # remove rows withstop words
    nltk.download('stopwords')
    df['text_clean'] = df['text_clean'].apply(lambda x: ' '.join([word for word in x.split() if word not in stopwords.words('english')]))

    return df



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['twitterpersona']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.6.3,<4.0.0',
 'nltk>=3.8.1,<4.0.0',
 'numpy>=1.24.1,<2.0.0',
 'pandas>=1.5.2,<2.0.0',
 'pillow>=9.4.0,<10.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'requests>=2.28.2,<3.0.0',
 'tweepy>=4.12.1,<5.0.0',
 'tweet-preprocessor>=0.6.0,<0.7.0',
 'urllib3>=1.26.14,<2.0.0',
 'wordcloud>=1.8.2.2,<2.0.0.0']

setup_kwargs = {
    'name': 'twitterpersona',
    'version': '0.3.5',
    'description': "Assess whether a twitter is positive or negative based on the user's recent tweets",
    'long_description': '[![ci-cd](https://github.com/UBC-MDS/twitter-persona/actions/workflows/ci-cd.yml/badge.svg)](https://github.com/UBC-MDS/twitter-persona/actions/workflows/ci-cd.yml) [![Documentation Status](https://readthedocs.org/projects/twitterpersona/badge/?version=latest)](https://twitterpersona.readthedocs.io/en/latest/?badge=latest) ![PyPI](https://img.shields.io/pypi/v/twitterpersona) [![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT) \n# twitterpersona\n\nTwitter is a popular social media app with over 1 billion user accounts. While a diversity of users is a strength, some individuals have concerns with the prevalence of "troll" accounts and individuals who exhibit unconstructive tone and diction whom they deem not worth engaging with.\nThe package twitterpersona is intended to provide insight into a twitter user based on their tweet history in effort to determine if an account is worth engaging with. The package provides an easy to use interface for determining the general sentiment expressed by a user.\n\n## Contributors and Maintainers\n- [Renzo Wijngaarden](https://github.com/Hawknum)\n- [Roan Raina](https://github.com/roanraina)\n- [Andy Wang](https://github.com/tiger12055)\n- [Yurui Feng](https://github.com/Yurui-Feng)\n\n\n## Quick Start\n\nTo get started with `twitterpersona`, install it using `pip`:\n\n```bash\n$ pip install twitterpersona\n```\nPlease visit the [documentation](https://twitterpersona.readthedocs.io/en/latest/?badge=latest) for more information and examples.\n\n## Classes and Functions\n1. `load_twitter_msg`: returns a user\'s recent tweets (as a dataframe) given their `user id` using the Twitter API.\n   1. `user_info()`: get user credentials details\n   2. `load_twitter_by_user()`: load specific user\'s tweets\n   3. `load_twitter_by_keywords()`: load specific keyword\'s tweets\n2. `sentiment_analysis`: determines the general (average) sentiment of recent tweets\n   1. `sentiment_labler()`: returns all tweets with the corresponding labels\n3. `preprocessing`: a spotter that identifies credit card numbers\n   1. `generalPreprocessing`: returns the processed tweet dataframe\n4. `generate_word_cloud`: a spotter that identifies credit card numbers\n   1. `create_wordcloud`: returns a matplotlib plot of the wordcloud\n\nBelow is a simple quick start example:\n\n```python\nfrom twitterpersona import load_twitter_msg, sentiment_analysis, preprocessing, generate_word_cloud\n\n# Create a cleanser, and don\'t add the default spotters\nuser = user_info(\'consumer key\', \'consumer secret\', \'access_token\', \'token_secret\')\ntwitter_df = load_twitter_by_user(\'someuser\', 30, user)\nsentiment_df = sentiment_labler(twitter_df, \'text\')\ncleaned_df = generalPreprocessing(sentiment_df)\nplt = generate_word_cloud(cleaned_df)\n```\nIn order to run test, you need to first install the vader_lexicon package\n\n```bash\n$ python -m nltk.downloader vader_lexicon\n```\n## Scope and Fit\n\nThere are existing packages that preform tweet analysis (including [twitter-sentiment-analysis](https://github.com/abdulfatir/twitter-sentiment-analysis), [tweetlytics](https://github.com/UBC-MDS/tweetlytics), and [pytweet](https://github.com/UBC-MDS/pytweet)). However, none of these packages focus of providing metrics in the context of determining if the twitter user is worth engaging with.\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines in CONTRIBUTING.md. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`twitterpersona` was created by Andy Wang, Renzo Wijngaarden, Roan Raina, Yurui Feng. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`twitterpersona` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Andy Wang, Renzo Wijngaarden, Roan Raina, Yurui Feng',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import nltk
from nltk.sentiment.vader import SentimentIntensityAnalyzer
import pandas as pd

nltk.download("vader_lexicon")

def sentiment_labler(df, col):
    """
    Labelling each row in a given column of tweets/text with positive, negative or neutral sentiment.

    Parameters
    ----------
    df : pd.DataFrame
        A dataframe that has been pre-processed.
    
    col : str
        Column name of the column containing tweets in the dataset.
    
    Returns
    -------

    df : pd.DataFrame
        Dataframe contains all tweets the corresponding labels.

    Examples
    --------
    sentiment_labler(df, "text")
    """
    sid = SentimentIntensityAnalyzer()

    def extract_sentiment(text):
        #Only three labels used by NLTK sentiment analyzer: neutral, positive and negative
        #each with a score
        #The sentiment is calculated based on the compound score with thresholds explained in:
        #https://towardsdatascience.com/social-media-sentiment-analysis-in-python-with-vader-no-training-required-4bc6a21e87b8
        scores = sid.polarity_scores(text)
        if scores["compound"] > 0.05:
            return "positive"
        elif scores["compound"] < -0.05:
            return "negative"
        else:
            return "neutral"

    labelled_df = df.assign(sentiment=df[col].apply(extract_sentiment))
    return labelled_df


def count_tweets(df, proportion= True):
    """
    Count the proportion of different sentiment tweets in a labelled sentiment dataframe

    Parameters
    ----------
    df : pd.DataFrame
        dataframe for each sentiment
    
    proportion : bool
        if True: returns the proportion; otherwise, return the counts.
    
    Returns
    -------
    dictionary
        A dictionary which calculates the proportion of three sentiments of tweets.

    Examples
    --------
    labelled_df = sentiment_labler(df, "text")
    count_tweets(labelled_df)
    """

    if proportion:
        sentiment_counts = df['sentiment'].value_counts(normalize = True)
    else:
        sentiment_counts = df['sentiment'].value_counts()

    return dict(sentiment_counts)


    